@@ -9,10 +9,10 @@ Most of the complexity of this example comes from the code turning the
 PDB information into a list of 3D positions, with associated scalar
 and connection information.
 
-We assign a scalar value for the atoms to differenciate the different
+We assign a scalar value for the atoms to differentiate the different
 types of atoms, but it does not correspond to the atomic mass. The size
 and the color of the atom on the visualization is therefore not
-chemicaly-significant.
+chemically-significant.
 
 The atoms are plotted using mlab.points3d, and connections between atoms
 are added to the dataset, and visualized using a surface module.
@@ -38,7 +38,7 @@ http://mmcif.pdb.org/dictionaries/pdb-correspondence/pdb2mmcif.html
 """
 
 # Author: Gael Varoquaux <gael.varoquaux@normalesup.org>
-# Copyright (c) 2008-2015, Enthought, Inc.
+# Copyright (c) 2008-2020, Enthought, Inc.
 # License: BSD Style.
 
 # The pdb code for the protein.
