@@ -2,17 +2,18 @@
 
 """
 # Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
-# Copyright (c) 2005-2008, Enthought, Inc.
+# Copyright (c) 2005-2020, Enthought, Inc.
 # License: BSD Style.
 
 import os.path
 import sys
 import subprocess
-
+import warnings
 
 # Enthought library imports.
 from traits.api import Instance, Range, Bool, Array, \
      Str, Property, Enum, Button
+from traits.etsconfig.api import ETSConfig
 from traitsui.api import FileEditor, auto_close_message
 from apptools.persistence import state_pickler
 from tvtk.api import tvtk
@@ -22,9 +23,29 @@ from mayavi.core.base import Base
 from mayavi.core.common import error
 
 from mayavi.core import lut
+
+
+# The directory that contains the pickled files for colormap
 lut_image_dir = os.path.dirname(lut.__file__)
-pylab_luts = state_pickler.load_state(os.path.join(lut_image_dir,
-                                                'pylab_luts.pkl'))
+pylab_luts_file = os.path.join(lut_image_dir, 'pylab_luts.pkl')
+
+try:
+    pylab_luts = state_pickler.load_state(pylab_luts_file)
+except (IOError, ValueError) as exception:
+    # IOError: failed to open file
+    # ValueError: pickled file is built from an OS w/ different
+    #             architecture, or with an incompatible protocol
+    message = ("Failed to load pylab colormaps from file:\n"
+               "{filepath}\n"
+               "Last error: {err_type} {err_message}\n"
+               "Some colormaps will not be available. "
+               "You may rebuild this file using the script provided "
+               "in the mayavi source code: scripts/cm2lut.py")
+    warnings.warn(message.format(filepath=pylab_luts_file,
+                                 err_type=type(exception).__name__,
+                                 err_message=str(exception)))
+    pylab_luts = {}
+
 
 #################################################################
 # Utility functions.
@@ -48,12 +69,12 @@ def check_lut_first_line(line, file_name=''):
         errmsg = "Error: The input data file \"%s\"\n"%(file_name)
         errmsg = errmsg+ "is not a proper lookup table file."\
                  " No LOOKUP_TABLE tag in first line. Try again."
-        raise IOError, errmsg
+        raise IOError(errmsg)
     try:
         n_color = first[2]
     except:
 
-        raise IOError, "Error: No size for LookupTable specified."
+        raise IOError("Error: No size for LookupTable specified.")
     else:
         return n_color
 
@@ -72,15 +93,16 @@ def parse_lut_file(file_name):
         if len(entr) != 4:
             errmsg="Error: insufficient or too much data in line "\
                     "-- \"%s\""%(entr)
-            raise IOError, errmsg
+            raise IOError(errmsg)
 
         tmp = []
         for color in entr:
             try:
                 tmp.append(float(color))
             except:
-                raise IOError, \
-                      "Unknown entry '%s'in lookup table input."%color
+                raise IOError(
+                    "Unknown entry '%s'in lookup table input."%color
+                )
         lut.append(tmp)
 
     return lut
@@ -90,7 +112,7 @@ def lut_mode_list():
     """ Function to generate the list of acceptable lut_mode values.
     """
     lut_mode_list = ( ['blue-red', 'black-white', 'file', ]
-                            + pylab_luts.keys() )
+                            + list(pylab_luts.keys()) )
     lut_mode_list.sort()
     return lut_mode_list
 
@@ -197,14 +219,14 @@ class LUTManager(Base):
 
         # Initialize the scalar bar.
         sc_bar = self.scalar_bar
-        sc_bar.set(lookup_table=self.lut,
-                   title=self.data_name,
-                   number_of_labels=self.number_of_labels,
-                   orientation='horizontal',
-                   width=0.8, height=0.17)
+        sc_bar.trait_set(lookup_table=self.lut,
+                         title=self.data_name,
+                         number_of_labels=self.number_of_labels,
+                         orientation='horizontal',
+                         width=0.8, height=0.17)
         pc = sc_bar.position_coordinate
-        pc.set(coordinate_system='normalized_viewport',
-               value=(0.1, 0.01, 0.0))
+        pc.trait_set(coordinate_system='normalized_viewport',
+                     value=(0.1, 0.01, 0.0))
         self._shadow_changed(self.shadow)
 
         # Initialize the lut.
@@ -219,8 +241,8 @@ class LUTManager(Base):
         ltp.on_trait_change(self.render)
 
         # Initialize the scalar_bar_widget
-        self.scalar_bar_widget.set(scalar_bar_actor=self.scalar_bar,
-                                   key_press_activation=False)
+        self.scalar_bar_widget.trait_set(scalar_bar_actor=self.scalar_bar,
+                                         key_press_activation=False)
         self._number_of_colors_changed(self.number_of_colors)
 
 
@@ -275,7 +297,7 @@ class LUTManager(Base):
             n_total = len(lut)
             n_color = self.number_of_colors
             if not n_color >= n_total:
-                lut = lut[::round(n_total/float(n_color))]
+                lut = lut[::int(round(n_total/float(n_color)))]
             self.load_lut_from_list(lut.tolist())
             #self.lut.force_build()
             return
@@ -298,10 +320,10 @@ class LUTManager(Base):
                 saturation_range = 0.0, 0.0
                 value_range = 0.0, 1.0
         lut = self.lut
-        lut.set(hue_range=hue_range, saturation_range=saturation_range,
-                value_range=value_range,
-                number_of_table_values=self.number_of_colors,
-                ramp='sqrt')
+        lut.trait_set(hue_range=hue_range, saturation_range=saturation_range,
+                      value_range=value_range,
+                      number_of_table_values=self.number_of_colors,
+                      ramp='sqrt')
         lut.modified()
         lut.force_build()
 
@@ -337,7 +359,7 @@ class LUTManager(Base):
             n_total = len(lut)
             if value > n_total:
                 return
-            lut = lut[::round(n_total/float(value))]
+            lut = lut[::int(round(n_total/float(value)))]
             self.load_lut_from_list(lut.tolist())
         else:
             lut = self.lut
@@ -408,6 +430,8 @@ class LUTManager(Base):
         self._default_data_range_changed(self.default_data_range)
 
     def _data_range_changed(self, value):
+        # should be guaranteed by callers, otherwise VTK will print an error
+        assert value[0] <= value[1]
         try:
             self.lut.set_range(value[0], value[1])
         except TypeError:
@@ -438,7 +462,7 @@ class LUTManager(Base):
                 f.close()
                 try:
                     lut_list = parse_lut_file(file_name)
-                except IOError, err_msg:
+                except IOError as err_msg:
                     msg = "Sorry could not parse LUT file: %s\n"%file_name
                     msg += err_msg
                     error(msg)
@@ -460,8 +484,9 @@ class LUTManager(Base):
 
     def _create_lut_fired(self):
         from tvtk import util
+        tk = 'wx' if ETSConfig.toolkit.lower().startswith('wx') else 'qt'
         script = os.path.join(os.path.dirname(util.__file__),
-                              'wx_gradient_editor.py')
+                              tk + '_gradient_editor.py')
         subprocess.Popen([sys.executable, script])
         auto_close_message('Launching LUT editor in separate process ...')
 
