@@ -5,8 +5,6 @@
 # Copyright (c) 2005, Enthought, Inc.
 # License: BSD Style.
 
-from distutils.version import StrictVersion
-
 # Enthought library imports.
 from traits.api import Instance, Range, Str, Bool, Property, \
                                     Float
@@ -18,8 +16,6 @@ from apptools.persistence import state_pickler
 from mayavi.core.module import Module
 from mayavi.core.pipeline_info import PipelineInfo
 
-VTK_VER = StrictVersion(tvtk.Version().vtk_version)
-
 
 ######################################################################
 # `Text` class.
@@ -70,22 +66,13 @@ class Text(Module):
     ########################################
     # The view of this object.
 
-    if VTK_VER > '5.1':
-        _text_actor_group = Group(Item(name='visibility'),
-                                  Item(name='text_scale_mode'),
-                                  Item(name='alignment_point'),
-                                  Item(name='minimum_size'),
-                                  Item(name='maximum_line_height'),
-                                  show_border=True,
-                                  label='Text Actor')
-    else:
-        _text_actor_group = Group(Item(name='visibility'),
-                                  Item(name='scaled_text'),
-                                  Item(name='alignment_point'),
-                                  Item(name='minimum_size'),
-                                  Item(name='maximum_line_height'),
-                                  show_border=True,
-                                  label='Text Actor')
+    _text_actor_group = Group(Item(name='visibility'),
+                              Item(name='text_scale_mode'),
+                              Item(name='alignment_point'),
+                              Item(name='minimum_size'),
+                              Item(name='maximum_line_height'),
+                              show_border=True,
+                              label='Text Actor')
 
     _position_group_2d = Group(Item(name='_x_position_2d',
                                     label='X position'),
@@ -154,16 +141,13 @@ class Text(Module):
         set the `actors` attribute up at this point.
         """
         actor = self.actor = tvtk.TextActor(input=str(self.text))
-        if VTK_VER > '5.1':
-            actor.set(text_scale_mode='prop', width=0.4, height=1.0)
-        else:
-            actor.set(scaled_text=True, width=0.4, height=1.0)
+        actor.trait_set(text_scale_mode='prop', width=0.4, height=1.0)
 
         c = actor.position_coordinate
-        c.set(coordinate_system='normalized_viewport',
+        c.trait_set(coordinate_system='normalized_viewport',
               value=(self.x_position, self.y_position, 0.0))
         c = actor.position2_coordinate
-        c.set(coordinate_system='normalized_viewport')
+        c.trait_set(coordinate_system='normalized_viewport')
 
         self._property.opacity = 1.0
 
@@ -230,7 +214,7 @@ class Text(Module):
                 y = 0
             elif y > 1:
                 y = 1
-            self.set(x_position=x, y_position=y,
+            self.trait_set(x_position=x, y_position=y,
                                                     trait_change_notify=False)
         self._shadow_positions(not value)
         self._change_position()
