@@ -21,7 +21,7 @@ class Vectors(Glyph):
     ######################################################################
     def setup_pipeline(self):
         super(Vectors, self).setup_pipeline()
-        self.glyph.set(scale_mode='scale_by_vector',
+        self.glyph.trait_set(scale_mode='scale_by_vector',
                        color_mode='color_by_vector',
                        show_scale_mode=False)
         self.glyph.glyph_source.glyph_position='tail'
