@@ -2,7 +2,7 @@
 """
 # Author:   R.Sreekanth <sreekanth [at] aero.iitb.ac.in>
 #               Suyog Dutt Jain <suyog.jain [at] aero.iitb.ac.in>
-# Copyright (c) 2009, Enthought, Inc.
+# Copyright (c) 2009-2020, Enthought, Inc.
 # License: BSD Style.
 
 from os.path import basename
@@ -13,7 +13,6 @@ from traitsui.api import View, Group, Item, Include
 from tvtk.api import tvtk
 
 # Local imports.
-from tvtk.common import is_old_pipeline
 from mayavi.core.file_data_source import FileDataSource
 from mayavi.core.pipeline_info import PipelineInfo
 from mayavi.core.common import error
@@ -71,7 +70,7 @@ class UnstructuredGridReader(FileDataSource):
     def get_output_object(self):
         """ Return the reader output port."""
         return self.reader.output_port
-    
+
     ######################################################################
     # Non-public interface
     ######################################################################
@@ -84,17 +83,18 @@ class UnstructuredGridReader(FileDataSource):
         extension = splitname[-1].lower()
         # Select UnstructuredGridreader based on file type
         old_reader = self.reader
-        if self._reader_dict.has_key(extension):
+        if extension in self._reader_dict:
             self.reader = self._reader_dict[extension]
         else:
             error('Invalid file extension for file: %s'%value)
             return
 
+        old_fname = self.reader.file_name
         self.reader.file_name = value.strip()
         self.reader.update_information()
         if isinstance(self.reader, tvtk.ExodusIIReader):
             # Make sure the point fields are read during Update().
-            for k in xrange(self.reader.number_of_point_result_arrays ):
+            for k in range(self.reader.number_of_point_result_arrays ):
                 arr_name = self.reader.get_point_result_array_name( k )
                 self.reader.set_point_result_array_status( arr_name, 1 )
         self.reader.update()
@@ -103,13 +103,9 @@ class UnstructuredGridReader(FileDataSource):
             old_reader.on_trait_change(self.render, remove=True)
         self.reader.on_trait_change(self.render)
 
-        old_outputs = self.outputs
-        if isinstance(self.reader, tvtk.ExodusIIReader):
-            self.outputs = [self.reader.output.get_block(0).get_block(0)]
-        else:
-            self.outputs = [self.reader.output]
+        self.outputs = [self.reader]
 
-        if self.outputs == old_outputs:
+        if old_fname != value:
             self.data_changed = True
 
         # Change our name on the tree view
@@ -130,15 +126,9 @@ class UnstructuredGridReader(FileDataSource):
 
     def __reader_dict_default(self):
         """Default value for reader dict."""
-        if is_old_pipeline():
-            rd = {'inp':tvtk.AVSucdReader(),
-                 'neu':tvtk.GAMBITReader(),
-                 'exii':tvtk.ExodusReader()
-                }
-        else:
-            rd = {'inp':tvtk.AVSucdReader(),
-                 'neu':tvtk.GAMBITReader(),
-                 'ex2':tvtk.ExodusIIReader()
-                }
+        rd = {'inp':tvtk.AVSucdReader(),
+              'neu':tvtk.GAMBITReader(),
+              'ex2':tvtk.ExodusIIReader()
+        }
 
         return rd
