@@ -13,8 +13,6 @@ from mayavi.sources.vtk_file_reader import VTKFileReader
 # Local imports.
 from mayavi.tests.common import get_example_data
 
-vtk_major_version = vtk.vtkVersion.GetVTKMajorVersion()
-vtk_minor_version = vtk.vtkVersion.GetVTKMinorVersion()
 
 class TestVTKFileReader(unittest.TestCase):
     def setUp(self):
@@ -29,7 +27,7 @@ class TestVTKFileReader(unittest.TestCase):
 
     def check(self, n_points, n_cells):
         """Do the actual testing."""
-        output = self.src.outputs[0]
+        output = self.src.get_output_dataset()
         self.assertEqual(output.number_of_points, n_points)
         self.assertEqual(output.number_of_cells, n_cells)
 
