@@ -10,10 +10,9 @@ seems no unique one-to-one VTK data array type to map it to.
 
 """
 # Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
-# Copyright (c) 2004-2008,  Enthought, Inc.
+# Copyright (c) 2004-2020,  Enthought, Inc.
 # License: BSD Style.
 
-import types
 import sys
 
 import vtk
@@ -26,8 +25,11 @@ except ImportError:
 import numpy
 
 # Enthought library imports.
-from tvtk.array_ext import set_id_type_array
-from tvtk.common import is_old_pipeline
+try:
+    from tvtk.array_ext import set_id_type_array
+    HAS_ARRAY_EXT = True
+except ImportError:
+    HAS_ARRAY_EXT = False
 
 # Useful constants for VTK arrays.
 VTK_ID_TYPE_SIZE = vtk.vtkIdTypeArray().GetDataTypeSize()
@@ -47,6 +49,45 @@ elif VTK_LONG_TYPE_SIZE == 8:
 BASE_REFERENCE_COUNT = vtk.vtkObject().GetReferenceCount()
 
 
+def getbuffer(array):
+    return getattr(numpy, 'getbuffer', memoryview)(array)
+
+
+def set_id_type_array_py(id_array, out_array):
+    """Given a 2D Int array (`id_array`), and a contiguous 1D numarray array
+    (`out_array`) having the correct size, this function sets the data from
+    `id_array` into `out_array` so that it can be used in place of a
+    `vtkIdTypeArray` in order to set the cells of a `vtkCellArray`.
+
+    Note that if `shape = id_array.shape` then `size(out_array) ==
+    shape[0]*(shape[1] + 1)` should be true. If not you'll get an
+    `AssertionError`.
+
+    `id_array` need not be contiguous but `out_array` must be.
+
+    """
+    assert numpy.issubdtype(id_array.dtype, numpy.signedinteger)
+    assert out_array.flags.contiguous == 1, \
+        "out_array must be contiguous."
+    shp = id_array.shape
+    assert len(shp) == 2, "id_array must be a two dimensional array."
+    sz = out_array.size
+    e_sz = shp[0]*(shp[1]+1)
+    assert sz == e_sz, \
+        "out_array size is incorrect, expected: %s, given: %s" % (e_sz, sz)
+
+    # we are guaranteed contiguous, so these just change the view (no copy)
+    out_shp = out_array.shape
+    out_array.shape = (shp[0], shp[1] + 1)
+    out_array[:, 0] = shp[1]
+    out_array[:, 1:] = id_array
+    out_array.shape = out_shp
+
+
+if not HAS_ARRAY_EXT:
+    set_id_type_array = set_id_type_array_py
+
+
 ######################################################################
 # The array cache.
 ######################################################################
@@ -70,7 +111,7 @@ class ArrayCache(object):
 
     def __contains__(self, vtk_arr):
         key = vtk_arr.__this__
-        return self._cache.has_key(key)
+        return key in self._cache
 
     ######################################################################
     # `ArrayCache` interface.
@@ -86,8 +127,9 @@ class ArrayCache(object):
         # `lambda` function is necessary because the callback will not
         # receive the object (it will receive `None`) and thus there
         # is no way to know which array reference one has to remove.
-        vtk_arr.AddObserver('DeleteEvent', lambda o, e, key=key: \
-                            self._remove_array(key))
+        vtk_arr.AddObserver(
+            'DeleteEvent', lambda o, e, key=key: self._remove_array(key)
+        )
 
         # Cache the array
         cache[key] = np_arr
@@ -119,7 +161,7 @@ class ArrayCache(object):
 _dummy = None
 # This makes the cache work even when the module is reloaded.
 for name in ['array_handler', 'tvtk.array_handler']:
-    if sys.modules.has_key(name):
+    if name in sys.modules:
         mod = sys.modules[name]
         if hasattr(mod, '_array_cache'):
             _dummy = mod._array_cache
@@ -133,29 +175,28 @@ else:
 del _dummy
 
 
-
-######################################################################
-# Array conversion functions.
-######################################################################
 def get_vtk_array_type(numeric_array_type):
     """Returns a VTK typecode given a numpy array."""
     # This is a Mapping from numpy array types to VTK array types.
-    _arr_vtk = {numpy.dtype(numpy.character):vtkConstants.VTK_UNSIGNED_CHAR,
-                numpy.dtype(numpy.uint8):vtkConstants.VTK_UNSIGNED_CHAR,
-                numpy.dtype(numpy.uint16):vtkConstants.VTK_UNSIGNED_SHORT,
-                numpy.dtype(numpy.int8):vtkConstants.VTK_CHAR,
-                numpy.dtype(numpy.int16):vtkConstants.VTK_SHORT,
-                numpy.dtype(numpy.int32):vtkConstants.VTK_INT,
-                numpy.dtype(numpy.uint32):vtkConstants.VTK_UNSIGNED_INT,
-                numpy.dtype(numpy.float32):vtkConstants.VTK_FLOAT,
-                numpy.dtype(numpy.float64):vtkConstants.VTK_DOUBLE,
-                numpy.dtype(numpy.complex64):vtkConstants.VTK_FLOAT,
-                numpy.dtype(numpy.complex128):vtkConstants.VTK_DOUBLE,
-                }
-    _extra = {numpy.dtype(ID_TYPE_CODE):vtkConstants.VTK_ID_TYPE,
-              numpy.dtype(ULONG_TYPE_CODE):vtkConstants.VTK_UNSIGNED_LONG,
-              numpy.dtype(LONG_TYPE_CODE):vtkConstants.VTK_LONG,
-             }
+    _arr_vtk = {
+        numpy.dtype('S'): vtkConstants.VTK_UNSIGNED_CHAR,  # numpy.character
+        numpy.dtype(numpy.uint8): vtkConstants.VTK_UNSIGNED_CHAR,
+        numpy.dtype(numpy.uint16): vtkConstants.VTK_UNSIGNED_SHORT,
+        numpy.dtype(numpy.int8): vtkConstants.VTK_CHAR,
+        numpy.dtype(numpy.int16): vtkConstants.VTK_SHORT,
+        numpy.dtype(numpy.int32): vtkConstants.VTK_INT,
+        numpy.dtype(numpy.uint32): vtkConstants.VTK_UNSIGNED_INT,
+        numpy.dtype(numpy.uint64): vtkConstants.VTK_UNSIGNED_LONG,
+        numpy.dtype(numpy.float32): vtkConstants.VTK_FLOAT,
+        numpy.dtype(numpy.float64): vtkConstants.VTK_DOUBLE,
+        numpy.dtype(numpy.complex64): vtkConstants.VTK_FLOAT,
+        numpy.dtype(numpy.complex128): vtkConstants.VTK_DOUBLE,
+    }
+    _extra = {
+        numpy.dtype(ID_TYPE_CODE): vtkConstants.VTK_ID_TYPE,
+        numpy.dtype(ULONG_TYPE_CODE): vtkConstants.VTK_UNSIGNED_LONG,
+        numpy.dtype(LONG_TYPE_CODE): vtkConstants.VTK_LONG,
+    }
     for t in _extra:
         if t not in _arr_vtk:
             _arr_vtk[t] = _extra[t]
@@ -166,22 +207,28 @@ def get_vtk_array_type(numeric_array_type):
         for key in _arr_vtk:
             if numpy.issubdtype(numeric_array_type, key):
                 return _arr_vtk[key]
-    raise TypeError, "Couldn't translate array's type to VTK"
+    raise TypeError(
+        "Couldn't translate array's type to VTK %s" % numeric_array_type
+    )
+
 
 def get_vtk_to_numeric_typemap():
     """Returns the VTK array type to numpy array type mapping."""
-    _vtk_arr = {vtkConstants.VTK_BIT:numpy.bool,
-                vtkConstants.VTK_CHAR:numpy.int8,
-                vtkConstants.VTK_UNSIGNED_CHAR:numpy.uint8,
-                vtkConstants.VTK_SHORT:numpy.int16,
-                vtkConstants.VTK_UNSIGNED_SHORT:numpy.uint16,
-                vtkConstants.VTK_INT:numpy.int32,
-                vtkConstants.VTK_UNSIGNED_INT:numpy.uint32,
-                vtkConstants.VTK_LONG:LONG_TYPE_CODE,
-                vtkConstants.VTK_UNSIGNED_LONG:ULONG_TYPE_CODE,
-                vtkConstants.VTK_ID_TYPE:ID_TYPE_CODE,
-                vtkConstants.VTK_FLOAT:numpy.float32,
-                vtkConstants.VTK_DOUBLE:numpy.float64}
+    _vtk_arr = {
+        vtkConstants.VTK_BIT: numpy.bool_,
+        vtkConstants.VTK_CHAR: numpy.int8,
+        vtkConstants.VTK_UNSIGNED_CHAR: numpy.uint8,
+        vtkConstants.VTK_SHORT: numpy.int16,
+        vtkConstants.VTK_UNSIGNED_SHORT: numpy.uint16,
+        vtkConstants.VTK_INT: numpy.int32,
+        vtkConstants.VTK_UNSIGNED_INT: numpy.uint32,
+        vtkConstants.VTK_LONG: LONG_TYPE_CODE,
+        vtkConstants.VTK_UNSIGNED_LONG: ULONG_TYPE_CODE,
+        vtkConstants.VTK_LONG_LONG: numpy.int64,
+        vtkConstants.VTK_ID_TYPE: ID_TYPE_CODE,
+        vtkConstants.VTK_FLOAT: numpy.float32,
+        vtkConstants.VTK_DOUBLE: numpy.float64
+    }
     return _vtk_arr
 
 
@@ -192,18 +239,21 @@ def get_numeric_array_type(vtk_array_type):
 
 def get_sizeof_vtk_array(vtk_array_type):
     """Returns the size of a VTK array type."""
-    _size_dict = {vtkConstants.VTK_BIT : 1,
-                  vtkConstants.VTK_CHAR : 1,
-                  vtkConstants.VTK_UNSIGNED_CHAR : 1,
-                  vtkConstants.VTK_SHORT : 2,
-                  vtkConstants.VTK_UNSIGNED_SHORT : 2,
-                  vtkConstants.VTK_INT : 4,
-                  vtkConstants.VTK_UNSIGNED_INT : 4,
-                  vtkConstants.VTK_LONG : VTK_LONG_TYPE_SIZE,
-                  vtkConstants.VTK_UNSIGNED_LONG : VTK_LONG_TYPE_SIZE,
-                  vtkConstants.VTK_ID_TYPE : VTK_ID_TYPE_SIZE,
-                  vtkConstants.VTK_FLOAT : 4,
-                  vtkConstants.VTK_DOUBLE : 8 }
+    _size_dict = {
+        vtkConstants.VTK_BIT: 1,
+        vtkConstants.VTK_CHAR: 1,
+        vtkConstants.VTK_UNSIGNED_CHAR: 1,
+        vtkConstants.VTK_SHORT: 2,
+        vtkConstants.VTK_UNSIGNED_SHORT: 2,
+        vtkConstants.VTK_INT: 4,
+        vtkConstants.VTK_UNSIGNED_INT: 4,
+        vtkConstants.VTK_LONG: VTK_LONG_TYPE_SIZE,
+        vtkConstants.VTK_UNSIGNED_LONG: VTK_LONG_TYPE_SIZE,
+        vtkConstants.VTK_LONG_LONG: 8,
+        vtkConstants.VTK_ID_TYPE: VTK_ID_TYPE_SIZE,
+        vtkConstants.VTK_FLOAT: 4,
+        vtkConstants.VTK_DOUBLE: 8
+    }
     return _size_dict[vtk_array_type]
 
 
@@ -246,7 +296,6 @@ def array2vtk(num_array, vtk_array=None):
           equivalent to each other.  For example if one is an integer
           array and the other a float.
 
-
     - vtk_array : `vtkDataArray` (default: `None`)
 
       If an optional `vtkDataArray` instance, is passed as an argument
@@ -259,11 +308,11 @@ def array2vtk(num_array, vtk_array=None):
 
     shape = z.shape
     assert len(shape) < 3, \
-           "Only arrays of dimensionality 2 or lower are allowed!"
-    assert not numpy.issubdtype(z.dtype, complex), \
-           "Complex numpy arrays cannot be converted to vtk arrays."\
-           "Use real() or imag() to get a component of the array before"\
-           " passing it to vtk."
+        "Only arrays of dimensionality 2 or lower are allowed!"
+    assert not numpy.issubdtype(z.dtype, numpy.complexfloating), \
+        "Complex numpy arrays cannot be converted to vtk arrays."\
+        "Use real() or imag() to get a component of the array before"\
+        " passing it to vtk."
 
     # First create an array of the right type by using the typecode.
     # Bit arrays need special casing.
@@ -296,7 +345,7 @@ def array2vtk(num_array, vtk_array=None):
 
     # Point the VTK array to the numpy data.  The last argument (1)
     # tells the array not to deallocate.
-    result_array.SetVoidArray(numpy.getbuffer(z_flat), len(z_flat), 1)
+    result_array.SetVoidArray(getbuffer(z_flat), len(z_flat), 1)
 
     if bit_array:
         # Handle bit arrays -- they have to be copied.  Note that bit
@@ -338,22 +387,24 @@ def vtk2array(vtk_array):
     """
     typ = vtk_array.GetDataType()
     assert typ in get_vtk_to_numeric_typemap().keys(), \
-           "Unsupported array type %s"%typ
+        "Unsupported array type %s" % typ
 
-    shape = vtk_array.GetNumberOfTuples(), \
-            vtk_array.GetNumberOfComponents()
+    shape = (vtk_array.GetNumberOfTuples(),
+             vtk_array.GetNumberOfComponents())
     if shape[0] == 0:
         dtype = get_numeric_array_type(typ)
         return numpy.array([], dtype)
 
-    # First check if this array already has a numpy array cached, if
-    # it does, reshape that and return it.
+    # First check if this array already has a numpy array cached,
+    # if it does and the array size has not been changed, reshape
+    # that and return it.
     if vtk_array in _array_cache:
         arr = _array_cache.get(vtk_array)
         if shape[1] == 1:
             shape = (shape[0], )
-        arr = numpy.reshape(arr, shape)
-        return arr
+        if arr.size == numpy.prod(shape):
+            arr = numpy.reshape(arr, shape)
+            return arr
 
     # If VTK's new numpy support is available, use the buffer interface.
     if numpy_support is not None and typ != vtkConstants.VTK_BIT:
@@ -383,33 +434,16 @@ def vtk2array(vtk_array):
     else:
         img_data.GetPointData().SetScalars(vtk_array)
 
-    if is_old_pipeline():
-        img_data.SetNumberOfScalarComponents(shape[1])
-        if typ == vtkConstants.VTK_ID_TYPE:
-            # Hack necessary because vtkImageData can't handle VTK_ID_TYPE.
-            img_data.SetScalarType(vtkConstants.VTK_LONG)
-            r_dtype = get_numeric_array_type(vtkConstants.VTK_LONG)
-        elif typ == vtkConstants.VTK_BIT:
-            img_data.SetScalarType(vtkConstants.VTK_CHAR)
-            r_dtype = get_numeric_array_type(vtkConstants.VTK_CHAR)
-        else:
-            img_data.SetScalarType(typ)
-            r_dtype = get_numeric_array_type(typ)
-        img_data.Update()
+    if typ == vtkConstants.VTK_ID_TYPE:
+        r_dtype = get_numeric_array_type(vtkConstants.VTK_LONG)
+    elif typ == vtkConstants.VTK_BIT:
+        r_dtype = get_numeric_array_type(vtkConstants.VTK_CHAR)
     else:
-        if typ == vtkConstants.VTK_ID_TYPE:
-            r_dtype = get_numeric_array_type(vtkConstants.VTK_LONG)
-        elif typ == vtkConstants.VTK_BIT:
-            r_dtype = get_numeric_array_type(vtkConstants.VTK_CHAR)
-        else:
-            r_dtype = get_numeric_array_type(typ)
-        img_data.Modified()
+        r_dtype = get_numeric_array_type(typ)
+    img_data.Modified()
 
     exp = vtk.vtkImageExport()
-    if is_old_pipeline():
-        exp.SetInput(img_data)
-    else:
-        exp.SetInputData(img_data)
+    exp.SetInputData(img_data)
 
     # Create an array of the right size and export the image into it.
     im_arr = numpy.empty((shape[0]*shape[1],), r_dtype)
@@ -477,7 +511,7 @@ def array2vtkCellArray(num_array, vtk_array=None):
     else:
         cells = vtk.vtkCellArray()
     assert cells.GetClassName() == 'vtkCellArray', \
-           'Second argument must be a `vtkCellArray` instance.'
+        'Second argument must be a `vtkCellArray` instance.'
 
     if len(num_array) == 0:
         return cells
@@ -509,10 +543,10 @@ def array2vtkCellArray(num_array, vtk_array=None):
     msg = "Invalid argument.  Valid types are a Python list of lists,"\
           " a Python list of numpy arrays, or a numpy array."
 
-    if issubclass(type(num_array), (types.ListType, types.TupleType)):
+    if issubclass(type(num_array), (list, tuple)):
         assert len(num_array[0]) > 0, "Input array must be 2D."
         tp = type(num_array[0])
-        if issubclass(tp, types.ListType): # Pure Python list.
+        if issubclass(tp, list):  # Pure Python list.
             _slow_array2cells(num_array, cells)
             return cells
         elif issubclass(tp, numpy.ndarray):  # List of arrays.
@@ -538,7 +572,7 @@ def array2vtkCellArray(num_array, vtk_array=None):
             _set_cells(cells, n_cells, id_typ_arr)
             return cells
         else:
-            raise TypeError, msg
+            raise TypeError(msg)
     elif issubclass(type(num_array), numpy.ndarray):
         assert len(num_array.shape) == 2, "Input array must be 2D."
         tmp_arr = _get_tmp_array(num_array)
@@ -548,7 +582,7 @@ def array2vtkCellArray(num_array, vtk_array=None):
         _set_cells(cells, shp[0], id_typ_arr)
         return cells
     else:
-        raise TypeError, msg
+        raise TypeError(msg)
 
 
 def array2vtkPoints(num_array, vtk_points=None):
@@ -572,7 +606,7 @@ def array2vtkPoints(num_array, vtk_points=None):
 
     """
     if vtk_points:
-        points  = vtk_points
+        points = vtk_points
     else:
         points = vtk.vtkPoints()
 
@@ -620,7 +654,7 @@ def array2vtkIdList(num_array, vtk_idlist=None):
 
 def is_array(arr):
     """Returns True if the passed `arr` is a numpy array or a List."""
-    if issubclass(type(arr), (numpy.ndarray, types.ListType)):
+    if issubclass(type(arr), (numpy.ndarray, list)):
         return True
     return False
 
@@ -647,7 +681,7 @@ def convert_array(arr, vtk_typ=None):
         elif vtk_typ.find('Array') > -1:
             try:
                 vtk_arr = getattr(vtk, vtk_typ)()
-            except TypeError: # vtk_typ == 'vtkDataArray'
+            except TypeError:  # vtk_typ == 'vtkDataArray'
                 return array2vtk(arr)
             else:
                 return array2vtk(arr, vtk_arr)
@@ -659,7 +693,7 @@ def convert_array(arr, vtk_typ=None):
 
 def is_array_sig(s):
     """Given a signature, return if the signature has an array."""
-    if not isinstance(s, basestring):
+    if not isinstance(s, str):
         return False
     arr_types = ['Array', 'vtkPoints', 'vtkIdList']
     for i in arr_types:
@@ -701,8 +735,8 @@ def get_correct_sig(args, sigs):
         if count == 0:
             # No sig has the right number of args.
             msg = "Insufficient number of arguments to method."\
-                  "Valid arguments are:\n%s"%sigs
-            raise TypeError, msg
+                  "Valid arguments are:\n%s" % sigs
+            raise TypeError(msg)
         elif count == 1:
             # If only one of the sigs has the right number of args,
             # return it.
@@ -710,7 +744,7 @@ def get_correct_sig(args, sigs):
         else:
             # More than one sig has the same number of args.
             # Check if args need conversion at all.
-            array_idx = [i for i, a in enumerate(args) \
+            array_idx = [i for i, a in enumerate(args)
                          if is_array_or_vtkarray(a)]
             n_arr = len(array_idx)
             if n_arr == 0:
