@@ -2,7 +2,7 @@
 Tests for array_handler.py.
 """
 # Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
-# Copyright (c) 2005-2008, Enthought, Inc.
+# Copyright (c) 2005-2020, Enthought, Inc.
 # License: BSD Style.
 
 import unittest
@@ -61,6 +61,8 @@ class TestArrayHandler(unittest.TestCase):
         #t_z.append(numpy.array([-128, 0, 127], numpy.character))
         t_z.append(numpy.array([-32768, 0, 32767], numpy.int16))
         t_z.append(numpy.array([-2147483648, 0, 2147483647], numpy.int32))
+        t_z.append(numpy.array([
+            -9223372036854775808, 0, 9223372036854775807], numpy.int64))
         t_z.append(numpy.array([0, 255], numpy.uint8))
         t_z.append(numpy.array([0, 65535], numpy.uint16))
         t_z.append(numpy.array([0, 4294967295], numpy.uint32))
@@ -95,10 +97,8 @@ class TestArrayHandler(unittest.TestCase):
             if len(z.shape) == 1:
                 self.assertEqual(len(z1.shape), 1)
             if z.dtype.char != 'c':
-                #print z1
                 self.assertEqual(sum(numpy.ravel(z) - numpy.ravel(z1)), 0)
             else:
-                #print z1.astype('c')
                 self.assertEqual(z, z1.astype('c'))
 
         # Check if type conversion works correctly.
@@ -162,17 +162,18 @@ class TestArrayHandler(unittest.TestCase):
 
         # Make sure the code at least runs for all the non-complex
         # numerical dtypes in numpy.
+        float_types = [x for x in numpy.sctypes['float']
+                       if x().dtype.name not in ('float16', 'float128')]
         for dtype in (numpy.sctypes['int'] + numpy.sctypes['uint'] +
-                            numpy.sctypes['float']):
+                      float_types):
             array_handler.array2vtk(numpy.zeros((1,), dtype=dtype))
 
-
     def test_arr2cell_array(self):
         """Test Numeric array to vtkCellArray conversion."""
         # Test list of lists.
         a = [[0], [1, 2], [3, 4, 5], [6, 7, 8, 9]]
         cells = array_handler.array2vtkCellArray(a)
-        z = numpy.array([1, 0, 2, 1,2, 3, 3,4,5, 4, 6,7,8,9])
+        z = numpy.array([1, 0, 2, 1, 2, 3, 3, 4, 5, 4, 6, 7, 8, 9])
         arr = array_handler.vtk2array(cells.GetData())
         self.assertEqual(numpy.sum(arr - z), 0)
         self.assertEqual(len(arr.shape), 1)
@@ -194,12 +195,12 @@ class TestArrayHandler(unittest.TestCase):
 
         # Test Numeric array handling.
         N = 3
-        a = numpy.zeros((N,3), numpy.int)
+        a = numpy.zeros((N,3), int)
         a[:,1] = 1
         a[:,2] = 2
         cells = array_handler.array2vtkCellArray(a)
         arr = array_handler.vtk2array(cells.GetData())
-        expect = numpy.array([3, 0, 1, 2]*3, numpy.int)
+        expect = numpy.array([3, 0, 1, 2]*3, int)
         self.assertEqual(numpy.alltrue(numpy.equal(arr, expect)),
                          True)
         self.assertEqual(cells.GetNumberOfCells(), N)
@@ -208,7 +209,7 @@ class TestArrayHandler(unittest.TestCase):
         l_a = [a[:,:1], a, a[:2,:2]]
         cells = array_handler.array2vtkCellArray(l_a)
         arr = array_handler.vtk2array(cells.GetData())
-        expect = numpy.array([1, 0]*3 + [3, 0, 1, 2]*3 + [2, 0,1]*2, numpy.int)
+        expect = numpy.array([1, 0]*3 + [3, 0, 1, 2]*3 + [2, 0,1]*2, int)
         self.assertEqual(numpy.alltrue(numpy.equal(arr, expect)),
                          True)
         self.assertEqual(cells.GetNumberOfCells(), N*2 + 2)
@@ -216,7 +217,7 @@ class TestArrayHandler(unittest.TestCase):
         # This should not take a long while.  This merely tests if a
         # million cells can be created rapidly.
         N = int(1e6)
-        a = numpy.zeros((N,3), numpy.int)
+        a = numpy.zeros((N,3), int)
         a[:,1] = 1
         a[:,2] = 2
         cells = array_handler.array2vtkCellArray(a)
@@ -314,7 +315,6 @@ class TestArrayHandler(unittest.TestCase):
                                   args[i], sigs[i])
             else:
                 s = array_handler.get_correct_sig(args[i], sigs[i])
-                #print s, res[i]
                 self.assertEqual(s, res[i])
 
     def test_deref_array(self):
