@@ -13,13 +13,12 @@ docs are shown.
 
 """
 # Author: Prabhu Ramachandran <prabhu [at] aero . iitb . ac . in>
-# Copyright (c) 2008,  Enthought, Inc.
+# Copyright (c) 2008-2020,  Enthought, Inc.
 # License: BSD Style.
 
 # Standard library imports.
-import vtk
-import types
 import inspect
+import sys
 
 # Enthought library imports.
 from traits.api import HasTraits, Property, List, Str, \
@@ -28,10 +27,15 @@ from traitsui.api import View, Group, Item, EnumEditor,\
                                     ListEditor, TextEditor
 from tvtk.api import tvtk
 from tvtk.common import get_tvtk_name
+import tvtk.vtk_module as vtk
 
-################################################################################
+# GLOBALS
+TVTK_CLASSES, TVTK_SOURCES, TVTK_FILTERS, TVTK_SINKS = None, None, None, None
+
+
+# ##############################################################################
 # Utility functions.
-################################################################################
+# ##############################################################################
 def get_tvtk_class_names():
     """Returns 4 lists:
 
@@ -49,6 +53,9 @@ def get_tvtk_class_names():
     w = o.GetGlobalWarningDisplay()
     o.SetGlobalWarningDisplay(0) # Turn it off.
 
+    old_stdout = sys.stdout
+    old_stderr = sys.stderr
+
     all = []
     src = []
     filter = []
@@ -58,6 +65,11 @@ def get_tvtk_class_names():
             klass = getattr(vtk, name)
             try:
                 c = klass()
+                # Some classes hijack sys.stdout/sys.stderr.
+                # Restore it when that happens.
+                if sys.stdout != old_stdout or sys.stderr != old_stderr:
+                    sys.stdout = old_stdout
+                    sys.stderr = old_stderr
             except (TypeError, NotImplementedError):
                 continue
 
@@ -87,31 +99,33 @@ def get_tvtk_class_names():
 
     return result
 
+
 def get_func_doc(func, fname):
     """Returns function documentation."""
     if inspect.isfunction(func):
         func_obj = func
     elif inspect.ismethod(func):
-        func_obj = func.im_func
+        func_obj = func.__func__
     else:
         return ''
-    args, vargs, vkw = inspect.getargs(func_obj.func_code)
-    defaults = func_obj.func_defaults
+    args, vargs, vkw = inspect.getargs(func_obj.__code__)
+    defaults = func_obj.__defaults__
     doc = fname + inspect.formatargspec(args, vargs, vkw, defaults)
     d = inspect.getdoc(func)
     if d is not None:
         doc += '\n\n' + d + '\n\n'
     return doc
 
+
 def get_tvtk_class_doc(obj):
     """Return's the objects documentation."""
     doc = obj.__doc__ + '\nTraits:\n-------------------\n\n'
 
     ignore = ['trait_added', 'trait_modified']
-    for key, trait in obj.traits().iteritems():
+    for key, trait in obj.traits().items():
         if key.startswith('_') or key.endswith('_') or key in ignore:
             continue
-        doc += '\n%s: %s'%(key, trait.help)
+        doc += '\n%s: %s'%(key, trait.tooltip or trait.desc or trait.help)
 
     doc += '\nMethods:\n----------------------\n\n'
     traits = obj.trait_names()
@@ -126,12 +140,37 @@ def get_tvtk_class_doc(obj):
 
     return doc
 
-# GLOBALS
-TVTK_CLASSES, TVTK_SOURCES, TVTK_FILTERS, TVTK_SINKS = get_tvtk_class_names()
 
-################################################################################
+def _setup_tvtk_names():
+    global TVTK_CLASSES, TVTK_SOURCES, TVTK_FILTERS, TVTK_SINKS
+    if TVTK_CLASSES is None:
+        r = get_tvtk_class_names()
+        TVTK_CLASSES, TVTK_SOURCES, TVTK_FILTERS, TVTK_SINKS = r
+
+
+def get_tvtk_classes():
+    _setup_tvtk_names()
+    return TVTK_CLASSES
+
+
+def get_tvtk_sources():
+    _setup_tvtk_names()
+    return TVTK_SOURCES
+
+
+def get_tvtk_filters():
+    _setup_tvtk_names()
+    return TVTK_FILTERS
+
+
+def get_tvtk_sinks():
+    _setup_tvtk_names()
+    return TVTK_SINKS
+
+
+# ##############################################################################
 # `DocSearch` class.
-################################################################################
+# ##############################################################################
 class DocSearch(object):
 
     """A simple class that provides a method to search through class
@@ -177,7 +216,7 @@ class DocSearch(object):
         ----------
             word -- name to search for.
         """
-        assert type(word) in types.StringTypes, \
+        assert type(word) is str, \
                "Sorry, passed argument, %s is not a string."%word
         if len(word.strip()) == 0:
             return []
@@ -282,7 +321,7 @@ class TVTKClassChooser(HasTraits):
     completions = List(Str)
 
     # List of available class names as strings.
-    available = List(TVTK_CLASSES)
+    available = List
 
     ########################################
     # Private traits.
@@ -320,7 +359,7 @@ class TVTKClassChooser(HasTraits):
                 width=800,
                 height=600,
                 title='TVTK class chooser',
-                buttons = ["OK", "Cancel"]
+                buttons=["OK", "Cancel"]
                 )
     ######################################################################
     # `object` interface.
@@ -375,24 +414,38 @@ class TVTKClassChooser(HasTraits):
             self.available = result
             self.completions = result[:self.n_completion]
 
+    def _available_default(self):
+        return get_tvtk_classes()
+
 
 ################################################################################
 # `TVTKSourceChooser` class.
 ################################################################################
 class TVTKSourceChooser(TVTKClassChooser):
-    available = List(TVTK_SOURCES)
+    available = List
 
-################################################################################
+    def _available_default(self):
+        return get_tvtk_sources()
+
+
+# ##############################################################################
 # `TVTKFilterChooser` class.
-################################################################################
+# ##############################################################################
 class TVTKFilterChooser(TVTKClassChooser):
-    available = List(TVTK_FILTERS)
+    available = List
 
-################################################################################
+    def _available_default(self):
+        return get_tvtk_filters()
+
+
+# ##############################################################################
 # `TVTKSinkChooser` class.
-################################################################################
+# ##############################################################################
 class TVTKSinkChooser(TVTKClassChooser):
-    available = List(TVTK_SINKS)
+    available = List
+
+    def _available_default(self):
+        return get_tvtk_sinks()
 
 
 def main():
@@ -402,5 +455,6 @@ def main():
     s = TVTKClassChooser()
     s.configure_traits()
 
+
 if __name__ == '__main__':
     main()
