@@ -2,21 +2,38 @@
 
 """
 # Author: Prabhu Ramachandran <prabhu_r@users.sf.net>
-# Copyright (c) 2004-2015,  Enthought, Inc.
+# Copyright (c) 2004-2020,  Enthought, Inc.
 # License: BSD Style.
 
-from __future__ import print_function
-
 import sys
 import weakref
 import os
 import logging
+from contextlib import contextmanager
 
 import vtk
 
 from traits import api as traits
 from . import messenger
 
+try:
+    from traits.api import PrefixMap, PrefixList
+except ImportError:  # use deprecated name
+    from traits.api import TraitPrefixMap, TraitPrefixList, Trait
+
+    class PrefixMap(Trait):
+        def __init__(self, map, *args, **kwargs):
+            if args:
+                args = (kwargs.pop('default_value'),) * args + (map,)
+            else:
+                args = (kwargs.pop('default_value'), map)
+            super().__init__(*args, **kwargs)
+
+    class PrefixList(Trait):
+        def __init__(self, list_, **kwargs):
+            super().__init__(kwargs['default_value'], TraitPrefixList(list_),
+                             **kwargs)
+
 # Setup a logger for this module.
 logger = logging.getLogger(__name__)
 
@@ -25,10 +42,12 @@ def BooleanEditor(*args, **kw):
     from traitsui.api import BooleanEditor as Editor
     return Editor(*args, **kw)
 
+
 def RGBColorEditor(*args, **kw):
     from traitsui.api import RGBColorEditor as Editor
     return Editor(*args, **kw)
 
+
 def FileEditor(*args, **kw):
     from traitsui.api import FileEditor as Editor
     return Editor(*args, **kw)
@@ -42,6 +61,7 @@ class TVTKObjectCache(weakref.WeakValueDictionary):
     def __init__(self, *args, **kw):
         self._observer_data = {}
         weakref.WeakValueDictionary.__init__(self, *args, **kw)
+
         def remove(wr, selfref=weakref.ref(self)):
             self = selfref()
             if self is not None:
@@ -139,62 +159,111 @@ true_bool_trait = traits.Trait('true',
                                {'true':  1, 't': 1, 'yes': 1,
                                 'y': 1, 'on': 1, 1: 1, 'false': 0,
                                 'f': 0, 'no':  0, 'n': 0,
-                                'off': 0, 0: 0, -1:0},
+                                'off': 0, 0: 0, -1: 0},
                                editor=BooleanEditor)
 
 false_bool_trait = traits.Trait('false', true_bool_trait)
 
 
-class TraitRevPrefixMap(traits.TraitPrefixMap):
-    """A reverse mapped TraitPrefixMap.  This handler allows for
+class RevPrefixMap(traits.TraitType):
+    """A reverse mapped PrefixMap.  This handler allows for
     something like the following::
 
       >>> class A(HasTraits):
-      ...     a = Trait('ab', TraitRevPrefixMap({'ab':1, 'cd':2}))
+      ...     a = RevPrefixMap({'ab':1, 'cd':2}, default_value='ab')
       ...
       >>> a = A()
       >>> a.a = 'c'
-      >>> print a.a
+      >>> print(a.a)
       'cd'
       >>> a.a = 1
-      >>> print a.a
+      >>> print(a.a)
       'ab'
 
     That is, you can set the trait to the value itself.  If multiple
     keys map to the same value, one of the valid keys will be used.
 
     """
-    def __init__(self, map):
-        traits.TraitPrefixMap.__init__(self, map)
-        self._rmap = {}
-        for key, value in map.items():
-            self._rmap[value] = key
+    is_mapped = True
+
+    def __init__(self, map, *extra_values, default_value=None, **kwargs):
+        self.map = dict(map)
+        if not self.map:
+            raise ValueError(
+                "The dictionary of valid values can not be empty."
+            )
+
+        self._map = {key: key for key in self.map}
+        self._rmap = {value: key for key, value in self.map.items()}
+
+        if default_value is None:
+            default_value = next(iter(self.map))
+        else:
+            default_value = self.value_for(default_value)
+
+        for key in extra_values:
+            self._rmap[key] = default_value
+
+        super().__init__(default_value, **kwargs)
+
+    def value_for(self, value):
+        if not isinstance(value, str):
+            raise traits.TraitError(
+                "Value must be {}, but a value {!r} was specified.".format(
+                    self.info(), value)
+            )
+
+        if value in self._map:
+            return self._map[value]
+
+        matches = [key for key in self.map if key.startswith(value)]
+        if len(matches) == 1:
+            self._map[value] = match = matches[0]
+            return match
+
+        raise traits.TraitError(
+            "Value must be {}, but a value {!r} was specified.".format(
+                self.info(), value)
+        )
 
     def validate(self, object, name, value):
         try:
             if value in self._rmap:
                 value = self._rmap[value]
-            if not value in self._map:
+            if value not in self._map:
                 match = None
-                n     = len( value )
+                n = len(value)
                 for key in self.map.keys():
                     if value == key[:n]:
                         if match is not None:
-                           match = None
-                           break
+                            match = None
+                            break
                         match = key
                 if match is None:
-                    self.error( object, name, value )
-                self._map[ value ] = match
-            return self._map[ value ]
+                    self.error(object, name, value)
+                self._map[value] = match
+            return self._map[value]
         except:
-            self.error( object, name, value )
+            self.error(object, name, value)
+
+    def mapped_value(self, value):
+        """ Get the mapped value for a value. """
+        return self.map[value]
+
+    def post_setattr(self, object, name, value):
+        setattr(object, name + "_", self.mapped_value(value))
 
     def info(self):
-        keys = [repr(x) for x in self._rmap.keys()]
-        keys.sort()
-        msg = ' or '.join(keys)
-        return traits.TraitPrefixMap.info(self) + ' or ' + msg
+        return (
+            " or ".join(sorted(repr(x) for x in self.map.keys()))
+            + ' (or any unique prefix) or '
+            + ' or '.join(sorted(repr(x) for x in self._rmap.keys()))
+        )
+
+    def get_editor(self, trait):
+        from traitsui.api import EnumEditor
+
+        return EnumEditor(values=self, cols=trait.cols or 3)
 
 
 def vtk_color_trait(default, **metadata):
@@ -235,6 +304,28 @@ vtk_file_prefix = traits.Trait(None, None, traits.Str, str,
 vtk_property_delegate = traits.Delegate('property', modify=True)
 
 
+_DISABLE_UPDATE = False
+
+
+@contextmanager
+def global_disable_update():
+    '''Disable updating any traits automatically due to changes in VTK.
+
+    Specifically, do not auto-update *any* objects due to the firing of a
+    ModifiedEvent from within VTK. This basically can call `update_traits` on
+    the object which can in turn fire off other callbacks.
+
+    Use this sparingly when you REALLY need to disable updates.
+
+    '''
+    global _DISABLE_UPDATE
+    try:
+        _DISABLE_UPDATE = True
+        yield
+    finally:
+        _DISABLE_UPDATE = False
+
+
 ######################################################################
 # Utility functions.
 ######################################################################
@@ -282,7 +373,8 @@ class TVTKBase(traits.HasStrictTraits):
     # Stores the names of the traits that need to be updated.
     _updateable_traits_ = traits.Tuple
 
-    # List of trait names that are to be included in the full traits view of this object.
+    # List of trait names that are to be included in the full traits view of
+    # this object.
     _full_traitnames_list_ = traits.List
 
     #################################################################
@@ -322,12 +414,11 @@ class TVTKBase(traits.HasStrictTraits):
         # Initialize the Python attribute.
         self._in_set = 0
         if obj:
-            assert obj.IsA(klass.__name__)
             self._vtk_obj = obj
         else:
             self._vtk_obj = klass()
 
-        # print "INIT", self.__class__.__name__, repr(self._vtk_obj)
+        # print("INIT", self.__class__.__name__)#, repr(self._vtk_obj))
 
         # Call the Super class to update the traits.
         # Inhibit any updates at this point since we update in the end
@@ -395,8 +486,8 @@ class TVTKBase(traits.HasStrictTraits):
     #################################################################
     # `HasTraits` interface.
     #################################################################
-
-    def class_trait_view_elements ( cls ):
+    @classmethod
+    def class_trait_view_elements(cls):
         """ Returns the ViewElements object associated with the class.
 
         The returned object can be used to access all the view elements
@@ -430,13 +521,11 @@ class TVTKBase(traits.HasStrictTraits):
             )
             for name in names:
                 if name in result:
-                    view_elements.content[ name ] = result[name]
+                    view_elements.content[name] = result[name]
         except Exception:
             pass
         return view_elements
 
-    class_trait_view_elements = classmethod( class_trait_view_elements )
-
     #################################################################
     # `TVTKBase` interface.
     #################################################################
@@ -465,7 +554,7 @@ class TVTKBase(traits.HasStrictTraits):
         the VTK observer callback functions.
 
         """
-        if self._in_set:
+        if self._in_set or _DISABLE_UPDATE:
             return
         if not hasattr(self, '_updateable_traits_'):
             return
@@ -539,11 +628,11 @@ class TVTKBase(traits.HasStrictTraits):
                 method(*val)
             else:
                 raise
-        self._in_set -= 1
+        finally:
+            self._in_set -= 1
         if force_update or self._wrapped_mtime(vtk_obj) > mtime:
             self.update_traits()
 
-
     def _wrap_call(self, vtk_method, *args):
         """This method allows us to safely call a VTK method without
         calling `update_traits` during the call.  This method is
@@ -566,8 +655,10 @@ class TVTKBase(traits.HasStrictTraits):
         vtk_obj = self._vtk_obj
         self._in_set += 1
         mtime = self._wrapped_mtime(vtk_obj) + 1
-        ret = vtk_method(*args)
-        self._in_set -= 1
+        try:
+            ret = vtk_method(*args)
+        finally:
+            self._in_set -= 1
         if self._wrapped_mtime(vtk_obj) > mtime:
             self.update_traits()
         return ret
